@@ -2,178 +2,178 @@ from __future__ import annotations
 
 import asyncio
 import base64
-import io
 import logging
 import re
 import zipfile
+from asyncio.futures import Future
 from contextlib import contextmanager
 from dataclasses import dataclass
-from typing import Any, Generator, IO, Literal, overload
+from functools import wraps
+from typing import IO, Any, Generator, Literal, overload, TYPE_CHECKING
 
 import nest_asyncio
-import paramiko.pkey
 
-from .parser import arglist
+from qslib.qs_is_protocol import CommandError
+from qslib.scpi_commands import AccessLevel, SCPICommand
+
 from .qsconnection_async import QSConnectionAsync
-from .tcprotocol import Protocol
-from .util import _unwrap_tags
-
-from sshtunnel import SSHTunnelForwarder
+from .protocol import Protocol
+from ._util import _unwrap_tags
 
 nest_asyncio.apply()
 
-from .base import RunStatus, MachineStatus, AccessLevel
+from .base import MachineStatus, RunStatus
 
 log = logging.getLogger(__name__)
 
+if TYPE_CHECKING:
+    from .experiment import Experiment
+    import matplotlib.pyplot as plt
+
+
+def _ensure_connection(level: AccessLevel = AccessLevel.Observer):
+    def wrap(func):
+        @wraps(func)
+        def wrapped(m: Machine, *args, **kwargs):
+            if m.automatic:
+                with m.ensured_connection(level):
+                    return func(m, *args, **kwargs)
+            else:
+                return func(m, *args, **kwargs)
+
+        return wrapped
+
+    return wrap
+
 
 @dataclass(init=False)
 class Machine:
     """
     A connection to a QuantStudio machine.  The connection can be opened and closed, and reused.
     A maximum access level can be set and changed, which will prevent the access level from going
-    above that level.  By default, the initial connection is as Observer.
+    above that level.
 
-    For clean access, the class provides a context manager for the connection, and the
-    `at_level` method provides a context manager for access level:
-
-    >>> with Machine('machine', 'password', max_access_level="Controller") as m:
-    >>>     # Now connected
-    >>>     print(m.run_status())  # runs at Observer level
-    >>>     with m.at_level("Controller", exclusive=True):
-    >>>         # Runs as Controller, and fails if another Controller is connected (exclusive)
-    >>>         m.abort_run()
-    >>>         m.drawer_open()
-    >>>     # Now back to Observer
-    >>>     print(m.status())
-    >>> # Now disconnected.
-
-    The connection context manager can also be used with :code:`with m:` form for a Machine
-    instance :code:`m` that already exists, in which case it will connect and disconnect.
-
-    If you don't want to use these, you can also use :any:`connect` and :any:`disconnect`.
-
-    Note that there is *no supported method* on the machine's server for removing hanging connections
-    other than a reboot, and AB's software will not start runs when other connections hold Controller
-    level.
+    By default, the class tries to handle connections and access automatically.
 
     Parameters
     ----------
 
-    host: str
+    host
         The host name or IP to connect to.
 
-    password: str
+    password
         The password to use. Note that this class does not obscure or protect the password at all,
         because it should not be relied on for security.  See :ref:`access-and-security`  for more
         information.
 
+    automatic
+        Whether or not to automatically handle connection, disconnection, and where possible,
+        access level.  Default True.
+
     max_access_level: "Observer", "Controller", "Administrator", or "Full"
         The maximum access level to allow.  This is *not* the initial access level, which
         will be Observer. The parameter can be changed later by changing the :code:`max_access_level`
         attribute.
 
-    port: int (default = 7000)
+    port
         The port to connect to. (Use the normal SCPI port, not the line-editor connection usually
-        on 2323).
-
-    tunnel_host: str or tuple[str, int], optional
-        If set, to a hostname/IP string or (hostname/IP, port) tuple, create an SSH tunnel to the tunnel_host
-        in order to connect to the machine, rather than connecting directly.  This uses paramiko, and
-        will not read your ssh configuration file for any host aliases, but will use your keys if you
-        have an ssh-agent running.
-
-    tunnel_user: str, optional
-        If set, specify the user for the tunnel connection.  If unset, the local user name is used.
-
-    tunnel_key: str or paramiko.pkey.PKey, optional
-        If set, specify the filename of a private key, or the paramiko-loaded key, to use for the tunnel
-        connection.
-
-    Examples
-    --------
-
-    Set up a connection
+        on 2323).  Default is 7000.
 
     """
 
     host: str
     password: str | None = None
-    max_access_level: AccessLevel | str = AccessLevel.Observer
+    automatic: bool = True
+    _max_access_level: AccessLevel = AccessLevel.Controller
     port: int = 7000
-    _initial_access_level: AccessLevel | str = AccessLevel.Observer
-    tunnel_host: str | tuple[str, int] | None = None
-    tunnel_user: str | None = None
-    tunnel_key: str | "paramiko.pkey.PKey" | None = None
+    _initial_access_level: AccessLevel = AccessLevel.Observer
+    _current_access_level: AccessLevel = AccessLevel.Guest
+    _connection: QSConnectionAsync | None = None
+
+    def asdict(self, password: bool = False) -> dict[str, str | int]:
+        d: dict[str, str | int] = {"host": self.host}
+        if self.password and password:
+            d["password"] = self.password
+        if self.max_access_level != Machine._max_access_level:
+            d["max_access_level"] = self.max_access_level.value
+        if self.port != Machine.port:
+            d["port"] = self.port
+        if self.automatic != Machine.automatic:
+            d["automatic"] = self.automatic
+
+        return d
+
+    @property
+    def connection(self) -> QSConnectionAsync:
+        """The :class:`QSConnectionAsync` for the connection, or a :class:`ConnectionError`."""
+        if self._connection is None:
+            raise ConnectionError
+        else:
+            return self._connection
+
+    @connection.setter
+    def connection(self, v: QSConnectionAsync | None) -> None:
+        self._connection = v
+
+    @property
+    def max_access_level(self) -> AccessLevel:
+        return self._max_access_level
+
+    @max_access_level.setter
+    def max_access_level(self, v: AccessLevel | str):
+        if not isinstance(v, AccessLevel):
+            self._max_access_level = AccessLevel(v)
+        else:
+            self._max_access_level = v
 
     def __init__(
         self,
         host: str,
         password: str | None = None,
-        max_access_level: AccessLevel | str = AccessLevel.Observer,
+        automatic: bool = True,
+        max_access_level: AccessLevel | str = AccessLevel.Controller,
         port: int = 7000,
-        connect_now: bool = False,
-        tunnel_host: str | tuple[str, int] | None = None,
-        tunnel_user: str | None = None,
-        tunnel_key: str | "paramiko.pkey.PKey" | None = None,
         _initial_access_level: AccessLevel | str = AccessLevel.Observer,
     ):
         self.host = host
         self.port = port
         self.password = password
+        self.automatic = automatic
         self.max_access_level = AccessLevel(max_access_level)
         self._initial_access_level = AccessLevel(_initial_access_level)
-        self.tunnel_host = tunnel_host
-        self.tunnel_user = tunnel_user
-        self.tunnel_key = tunnel_key
-        self._tunnel: SSHTunnelForwarder | None = None
-        self._qsc: QSConnectionAsync | None = None
-
-        if connect_now:
-            self.connect()
-
-    @property
-    def _use_tunnel(self) -> bool:
-        return self.tunnel_host is not None
+        self._connection = None
 
     def connect(self) -> None:
-        """Open the connection."""
+        """Open the connection manually."""
         loop = asyncio.get_event_loop()
 
-        if self._use_tunnel:
-            self._tunnel = SSHTunnelForwarder(
-                self.tunnel_host,
-                ssh_username=self.tunnel_user,
-                ssh_pkey=self.tunnel_key,
-                remote_bind_address=(self.host, self.port),
-            )
-            self._tunnel.start()
-            port = self._tunnel.local_bind_port
-            host = "localhost"
-        else:
-            port = self.port
-            host = self.host
-
-        self._qsc = QSConnectionAsync(
-            host,
-            port,
+        self.connection = QSConnectionAsync(
+            self.host,
+            self.port,
             password=self.password,
-            initial_access_level=AccessLevel(self._initial_access_level),
+            initial_access_level=self._initial_access_level,
         )
-        loop.run_until_complete(self._qsc.connect())
+        loop.run_until_complete(self.connection.connect())
+        self._current_access_level = self.get_access_level()[0]
 
     @property
-    def connected(self):
-        if self._qsc is None:
+    def connected(self) -> bool:
+        """Whether or not there is a current connection to the machine.
+
+        Note that when using automatic connections, this will usually be False,
+        because connections will only be active when running a command.
+        """
+        if (not hasattr(self, "_connection")) or (self._connection is None):
             return False
         else:
-            return self._qsc.connected
+            return self.connection.connected
 
     def __enter__(self) -> Machine:
         self.connect()
         return self
 
-    def run_command(self, command: str) -> str:
+    @_ensure_connection(AccessLevel.Guest)
+    def run_command(self, command: str | SCPICommand) -> str:
         """Run a SCPI command, and return the response as a string.
         Waits for OK, not just NEXT.
 
@@ -192,19 +192,24 @@ class Machine:
         CommandError
             Received an Error response.
         """
-        if self._qsc is None:
+        if self.connection is None:
             raise ConnectionError(f"Not connected to {self.host}.")
         loop = asyncio.get_event_loop()
-        return loop.run_until_complete(self._qsc.run_command(command))
+        try:
+            return loop.run_until_complete(self.connection.run_command(command))
+        except CommandError as e:
+            e.__traceback__ = None
+            raise e
 
-    def run_command_to_ack(self, command: str) -> str:
+    @_ensure_connection(AccessLevel.Guest)
+    def run_command_to_ack(self, command: str | SCPICommand) -> str:
         """Run an SCPI command, and return the response as a string.
         Returns after the command is processed (OK or NEXT), but potentially
         before it has completed (NEXT).
 
         Parameters
         ----------
-        command : str
+        commands
             command to run
 
         Returns
@@ -217,22 +222,58 @@ class Machine:
         CommandError
             Received an Error response.
         """
-        if self._qsc is None:
+        if self.connection is None:
             raise ConnectionError(f"Not connected to {self.host}")
         loop = asyncio.get_event_loop()
-        return loop.run_until_complete(self._qsc.run_command(command, just_ack=True))
+        try:
+            return loop.run_until_complete(
+                self.connection.run_command(command, just_ack=True)
+            )
+        except CommandError as e:
+            e.__traceback__ = None
+            raise e
 
+    @_ensure_connection(AccessLevel.Guest)
+    def run_command_bytes(self, command: str | bytes | SCPICommand) -> bytes:
+        """Run an SCPI command, and return the response as bytes (undecoded).
+        Returns after the command is processed (OK or NEXT), but potentially
+        before it has completed (NEXT).
+
+        Parameters
+        ----------
+        command
+            command to run
+
+        Returns
+        -------
+        bytes
+            Response message (after "OK" or "NEXT", likely "" in latter case)
+
+        Raises
+        ------
+        CommandError
+            Received
+        """
+        if self.connection is None:
+            raise ConnectionError(f"Not connected to {self.host}.")
+        if isinstance(command, str):
+            command = command.encode()
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(self.connection._protocol.run_command(command))
+
+    @_ensure_connection(AccessLevel.Controller)
     def define_protocol(self, protocol: Protocol) -> None:
         """Send a protocol to the machine. This *is not related* to a particular
         experiment.  The name on the machine is set by the protocol.
 
         Parameters
         ----------
-        protocol : Protocol
+        protocol
             protocol to send
         """
-        self.run_command(f"{protocol.to_command()}")
+        self.run_command(protocol.to_scpicommand())
 
+    @_ensure_connection(AccessLevel.Observer)
     def read_dir_as_zip(self, path: str, leaf: str = "FILE") -> zipfile.ZipFile:
         """Read a directory on the
 
@@ -248,9 +289,8 @@ class Machine:
         zipfile.ZipFile
             the returned zip file
         """
-        x = self.run_command_bytes(f"{leaf}:ZIPREAD? {path}")
-
-        return zipfile.ZipFile(io.BytesIO(base64.decodebytes(x[7:-10])))
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(self.connection.read_dir_as_zip(path, leaf))
 
     @overload
     def list_files(
@@ -274,6 +314,7 @@ class Machine:
     ) -> list[str]:
         ...
 
+    @_ensure_connection(AccessLevel.Observer)
     def list_files(
         self,
         path: str,
@@ -282,39 +323,15 @@ class Machine:
         verbose: bool = False,
         recursive: bool = False,
     ) -> list[str] | list[dict[str, Any]]:
-        if not verbose:
-            if recursive:
-                raise NotImplementedError
-            return self.run_command(f"{leaf}:LIST? {path}").split("\n")[1:-1]
-        else:
-            v = self.run_command(f"{leaf}:LIST? -verbose {path}").split("\n")[1:-1]
-            ret = []
-            for x in v:
-                rm = re.match(
-                    r'"([^"]+)" -type=(\S+) -size=(\S+) -mtime=(\S+) -atime=(\S+) -ctime=(\S+)$',
-                    x,
-                )
-                if rm is None:
-                    x = arglist.parseString(x)
-                    d = {}
-                    d["path"] = x["arglist"]["args"][0]  # type: ignore
-                    d |= x["arglist"]["opts"]  # type: ignore
-                else:
-                    d = {}
-                    d["path"] = rm.group(1)
-                    d["type"] = rm.group(2)
-                    d["size"] = int(rm.group(3))
-                    d["mtime"] = float(rm.group(4))
-                    d["atime"] = float(rm.group(5))
-                    d["ctime"] = float(rm.group(6))
-                if d["type"] == "folder" and recursive:
-                    ret += self.list_files(
-                        d["path"], leaf=leaf, verbose=True, recursive=True
-                    )
-                else:
-                    ret.append(d)
-            return ret
 
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(
+            self.connection.list_files(
+                path, leaf=leaf, verbose=verbose, recursive=recursive
+            )
+        )
+
+    @_ensure_connection(AccessLevel.Observer)
     def read_file(
         self, path: str, context: str | None = None, leaf: str = "FILE"
     ) -> bytes:
@@ -333,16 +350,11 @@ class Machine:
         bytes
             returned file
         """
-        if not context:
-            contexts = ""
-        elif context[-1] == ":":
-            contexts = context
-        else:
-            contexts = context + ":"
-        x = self.run_command_bytes(f"{leaf}:READ? {contexts}{path}")
-
-        return base64.decodebytes(x[7:-10])
+        return asyncio.get_event_loop().run_until_complete(
+            self.connection.read_file(path, context, leaf)
+        )
 
+    @_ensure_connection(AccessLevel.Controller)
     def write_file(self, path: str, data: str | bytes) -> None:
         if isinstance(data, str):
             data = data.encode()
@@ -355,6 +367,7 @@ class Machine:
             + b"\n</quote.base64>"
         )
 
+    @_ensure_connection(AccessLevel.Observer)
     def list_runs_in_storage(self) -> list[str]:
         """List runs in machine storage.
 
@@ -367,8 +380,11 @@ class Machine:
         """
         x = self.run_command("FILE:LIST? public_run_complete:")
         a = x.split("\n")[1:-1]
-        return [re.sub("^public_run_complete:", "", s) for s in a if s.endswith("eds")]
+        return [
+            re.sub("^public_run_complete:", "", s)[:-4] for s in a if s.endswith(".eds")
+        ]
 
+    @_ensure_connection(AccessLevel.Observer)
     def load_run_from_storage(self, path: str) -> "Experiment":  # type: ignore
         from .experiment import Experiment
 
@@ -376,6 +392,7 @@ class Machine:
         """
         return Experiment.from_machine_storage(self, path)
 
+    @_ensure_connection(AccessLevel.Guest)
     def save_run_from_storage(
         self, machine_path: str, download_path: str | IO[bytes], overwrite: bool = False
     ) -> None:
@@ -407,43 +424,22 @@ class Machine:
             finally:
                 file.close()
 
-    def run_command_bytes(self, command: str | bytes) -> bytes:
-        """Run an SCPI command, and return the response as bytes (undecoded).
-        Returns after the command is processed (OK or NEXT), but potentially
-        before it has completed (NEXT).
-
-        Parameters
-        ----------
-        command : str | bytes
-            command to run
-
-        Returns
-        -------
-        bytes
-            Response message (after "OK" or "NEXT", likely "" in latter case)
-
-        Raises
-        ------
-        CommandError
-            Received
-        """
-        if self._qsc is None:
-            raise ConnectionError(f"Not connected to {self.host}.")
-        if isinstance(command, str):
-            command = command.encode()
-        loop = asyncio.get_event_loop()
-        return loop.run_until_complete(self._qsc._protocol.run_command(command))
-
+    @_ensure_connection(AccessLevel.Observer)
     def _get_log_from_byte(self, name: str | bytes, byte: int) -> bytes:
-        logfuture = asyncio.Future()
-        if self._qsc is None:
+        logfuture: Future[
+            tuple[bytes, bytes, Future[tuple[bytes, bytes, None]] | None]
+        ] = asyncio.Future()
+        if self.connection is None:
             raise Exception
         if isinstance(name, bytes):
             name = name.decode()
-        self._qsc._protocol.waiting_commands.append((b"logtransfer", logfuture))
+        self.connection._protocol.waiting_commands.append((b"logtransfer", logfuture))
 
-        logcommand = self._qsc._protocol.run_command(
-            f"eval? session.writeQueue.put(('OK logtransfer \\<quote.base64\\>\\\\n' + (lambda x: [x.seek({byte}), __import__('base64').encodestring(x.read())][1])(open('/data/vendor/IS/experiments/{name}/apldbio/sds/messages.log')) + '\\</quote.base64\\>\\\\n', None))",
+        logcommand = self.connection._protocol.run_command(
+            f"eval? session.writeQueue.put(('OK logtransfer \\<quote.base64\\>\\\\n'"
+            f" + (lambda x: [x.seek({byte}), __import__('base64').encodestring(x.read())][1])"
+            f"(open('/data/vendor/IS/experiments/{name}/apldbio/sds/messages.log')) +"
+            " '\\</quote.base64\\>\\\\n', None))",
             ack_timeout=200,
         )
 
@@ -455,28 +451,30 @@ class Machine:
 
         return base64.decodebytes(logfuture.result()[1][15:-17])
 
+    @_ensure_connection(AccessLevel.Observer)
     def run_status(self) -> RunStatus:
         """Return information on the status of any run."""
         return RunStatus.from_machine(self)
 
+    @_ensure_connection(AccessLevel.Observer)
     def machine_status(self) -> MachineStatus:
         """Return information on the status of the machine."""
         return MachineStatus.from_machine(self)
 
+    @_ensure_connection(AccessLevel.Observer)
     def get_running_protocol(self) -> Protocol:
         p = _unwrap_tags(self.run_command("PROT? ${Protocol}"))
         pn, svs, rm = self.run_command(
             "RET ${Protocol} ${SampleVolume} ${RunMode}"
         ).split()
-        p = f"PROT -volume={svs} -runmod={rm} {pn} " + p
-        return Protocol.from_command(p)
+        p = f"PROT -volume={svs} -runmode={rm} {pn} " + p
+        return Protocol.from_scpicommand(SCPICommand.from_string(p))
 
     def set_access_level(
         self,
         access_level: AccessLevel | str,
         exclusive: bool = False,
         stealth: bool = False,
-        _log: bool = True,
     ) -> None:
         access_level = AccessLevel(access_level)
 
@@ -489,8 +487,8 @@ class Machine:
         self.run_command(
             f"ACC -stealth={stealth} -exclusive={exclusive} {access_level}"
         )
-        if _log:
-            log.info(f"Took access level {access_level} {exclusive=} {stealth=}")
+        log.debug(f"Took access level {access_level} {exclusive=} {stealth=}")
+        self._current_access_level = access_level
 
     def get_access_level(
         self,
@@ -499,8 +497,20 @@ class Machine:
         m = re.match(r"^-stealth=(\w+) -exclusive=(\w+) (\w+)", ret)
         if m is None:
             raise ValueError(ret)
-        return AccessLevel(m[3]), m[2] == "True", m[1] == "True"
+        level = AccessLevel(m[3])
+        self._current_access_level = level
+        return level, m[2] == "True", m[1] == "True"
 
+    @property
+    def access_level(self) -> AccessLevel:
+        return self._current_access_level
+
+    @access_level.setter
+    def access_level(self, v: AccessLevel | str) -> None:
+        with self.ensured_connection(AccessLevel.Guest):
+            self.set_access_level(v)
+
+    @_ensure_connection(AccessLevel.Controller)
     def drawer_open(self) -> None:
         """Open the machine drawer using the OPEN command. This will ensure proper
         cover/drawer operation.  It *will not check run status*, and will open and
@@ -508,13 +518,14 @@ class Machine:
         """
         self.run_command("OPEN")
 
-    def drawer_close(self, lower_cover: bool = False) -> None:
+    @_ensure_connection(AccessLevel.Controller)
+    def drawer_close(self, lower_cover: bool = True) -> None:
         """Close the machine drawer using the OPEN command. This will ensure proper
         cover/drawer operation.  It *will not check run status*, and will open and
         close the drawer during runs and potentially during imaging.
 
-        By default, it will not lower the cover automaticaly after closing, use
-        lower_cover=True to do so.
+        By default, it will lower the cover automaticaly after closing, use
+        lower_cover=False to not do so.
         """
         self.run_command("CLOSE")
         if lower_cover:
@@ -523,21 +534,25 @@ class Machine:
     @property
     def status(self) -> RunStatus:
         """Return the current status of the run."""
-        return RunStatus.from_machine(self)
+        with self.ensured_connection(AccessLevel.Observer):
+            return RunStatus.from_machine(self)
 
     @property
     def drawer_position(self) -> Literal["Open", "Closed", "Unknown"]:
         """Return the drawer position from the DRAW? command."""
-        return self.run_command("DRAW?")  # type: ignore
+        with self.ensured_connection(AccessLevel.Observer):
+            return self.run_command("DRAW?")  # type: ignore
 
     @property
     def cover_position(self) -> Literal["Up", "Down", "Unknown"]:
         """Return the cover position from the ENG? command. Note that
         this does not always seem to work."""
-        f = self.run_command("ENG?")
-        assert f in ["Up", "Down", "Unknown"]
-        return f  # type: ignore
+        with self.ensured_connection(AccessLevel.Observer):
+            f = self.run_command("ENG?")
+            assert f in ["Up", "Down", "Unknown"]
+            return f  # type: ignore
 
+    @_ensure_connection(AccessLevel.Controller)
     def cover_lower(self) -> None:
         """Lower/engage the plate cover, closing the drawer if needed."""
         self.drawer_close(lower_cover=False)
@@ -547,38 +562,40 @@ class Machine:
         self.disconnect()
 
     def __del__(self) -> None:
-        if self._qsc is not None:
+        if self.connected:
             self.disconnect()
 
     def disconnect(self) -> None:
         """Cleanly disconnect from the machine."""
-        if self._qsc is None:
+        if self.connection is None:
             raise ConnectionError(f"Not connected to {self.host}.")
 
         loop = asyncio.get_event_loop()
 
-        loop.run_until_complete(self._qsc.disconnect())
-        self._qsc = None
-
-        if self._tunnel is not None:
-            self._tunnel.stop()
-            self._tunnel = None
+        loop.run_until_complete(self.connection.disconnect())
+        self._connection = None
+        self._current_access_level = AccessLevel.Guest
 
+    @_ensure_connection(AccessLevel.Controller)
     def abort_current_run(self) -> None:
         """Abort (stop immediately) the current run."""
         self.run_command("AbortRun ${RunTitle}")
 
+    @_ensure_connection(AccessLevel.Controller)
     def stop_current_run(self) -> None:
         """Stop (stop after cycle end) the current run."""
         self.run_command("StopRun ${RunTitle}")
 
+    @_ensure_connection(AccessLevel.Controller)
     def pause_current_run(self) -> None:
         """Pause the current run now."""
         self.run_command_to_ack("PAUSe")
 
+    @_ensure_connection(AccessLevel.Controller)
     def pause_current_run_at_temperature(self) -> None:
         raise NotImplementedError
 
+    @_ensure_connection(AccessLevel.Controller)
     def resume_current_run(self) -> None:
         """Resume the current run."""
         self.run_command_to_ack("RESume")
@@ -591,30 +608,38 @@ class Machine:
         the lamp, temperature control, etc.  It will do so even if there is
         currently a run.
         """
-        s = self.run_command("POW?").lower()
-        if s == "on":
-            return True
-        elif s == "off":
-            return False
-        else:
-            raise ValueError(f"Unexpected power status: {s}")
+        with self.ensured_connection(AccessLevel.Observer):
+            s = self.run_command("POW?").lower()
+            if s == "on":
+                return True
+            elif s == "off":
+                return False
+            else:
+                raise ValueError(f"Unexpected power status: {s}")
 
     @power.setter
-    def power(self, value: Literal["on", "off", True, False]):  # type: ignore
-        if value is True:
-            value = "on"
-        elif value is False:
-            value = "off"
-        self.run_command(f"POW {value}")
+    def power(self, value: Literal["on", "off", True, False]):
+        with self.ensured_connection(AccessLevel.Controller):
+            if value is True:
+                value = "on"
+            elif value is False:
+                value = "off"
+            self.run_command(f"POW {value}")
 
     @property
     def current_run_name(self) -> str | None:
         """Name of current run, or None if no run is active."""
-        out = self.run_command("RUNTitle?")
-        if out == "-":
-            return None
-        else:
-            return out
+        with self.ensured_connection(AccessLevel.Observer):
+            out = self.run_command("RUNTitle?")
+            if out == "-":
+                return None
+            else:
+                return re.sub(r"(<([\w.]+)>)?([^<]+)(</[\w.]+>)?", r"\3", out)
+
+    @_ensure_connection(AccessLevel.Controller)
+    def restart_system(self) -> None:
+        """Restart the system (both the InstrumentServer and android interface) by killing the zygote process."""
+        self.run_command(SCPICommand("SYST:EXEC", "killall zygote"))
 
     @contextmanager
     def at_access(
@@ -624,10 +649,29 @@ class Machine:
         stealth: bool = False,
     ) -> Generator[Machine, None, None]:
         fac, fex, fst = self.get_access_level()
-        self.set_access_level(access_level, exclusive, stealth, _log=False)
-        log.info(f"Took access level {access_level} {exclusive=} {stealth=}.")
+        self.set_access_level(access_level, exclusive, stealth)
+        log.debug(f"Took access level {access_level} {exclusive=} {stealth=}.")
         yield self
-        self.set_access_level(fac, fex, fst, _log=False)
-        log.info(
+        self.set_access_level(fac, fex, fst)
+        log.debug(
             f"Dropped access level {access_level}, returning to {fac} exclusive={fex} stealth={fst}."
         )
+
+    @contextmanager
+    def ensured_connection(
+        self, access_level: AccessLevel = AccessLevel.Observer
+    ) -> Generator[Machine, None, None]:
+        if self.automatic:
+            was_connected = self.connected
+            if not was_connected:
+                self.connect()
+            old_access = self.access_level
+            if old_access < access_level:
+                self.set_access_level(access_level)
+            yield self
+            if not was_connected:
+                self.disconnect()
+            elif old_access < access_level:
+                self.set_access_level(old_access)
+        else:
+            yield self
