@@ -4,7 +4,7 @@ from __future__ import annotations
 import xml.etree.ElementTree as ET
 from dataclasses import dataclass, field
 from io import BytesIO
-from typing import Iterable, Mapping, Optional, Sequence, Tuple, List, Dict, Union
+from typing import Any, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, Union
 from uuid import uuid1
 
 import numpy as np
@@ -44,9 +44,9 @@ class Sample:
     @classmethod
     def from_platesetup_sample(cls, se: ET.Element) -> Sample:  # type: ignore
         return cls(
-            se.findtext("Name"),
-            se.findtext("CustomProperty/Property[.='SP_UUID']/../Value"),
-            _process_color_from_str_int(se.findtext("Color")),
+            se.findtext("Name") or "Unnamed",
+            se.findtext("CustomProperty/Property[.='SP_UUID']/../Value") or uuid1().hex,
+            _process_color_from_str_int(se.findtext("Color") or "-1"),
         )
 
     def to_xml(self) -> ET.Element:
@@ -78,8 +78,12 @@ class PlateSetup:
         sample_wells: Dict[str, List[str]] = dict()
 
         for fv in sample_fvs:
-            idx = int(fv.find("Index").text)
-            sample = Sample.from_platesetup_sample(fv.find("FeatureItem/Sample"))
+            if x := fv.findtext("Index"):
+                idx = int(x)
+            else:
+                raise ValueError
+            if y := fv.find("FeatureItem/Sample"):
+                sample = Sample.from_platesetup_sample(y)
             if sample.name in samples_by_name.keys():
                 assert sample == samples_by_name[sample.name]
                 assert sample == samples_by_uuid[sample.uuid]
@@ -143,10 +147,10 @@ class PlateSetup:
 
         return wells
 
-    def well_samples_as_array(self) -> np.ndarray:
+    def well_samples_as_array(self) -> np.ndarray[Any, Any]:
         return self.well_sample.to_numpy().reshape((8, 12))
 
-    def to_lineprotocol(self, timestamp, run_name=None):
+    def to_lineprotocol(self, timestamp: int, run_name: str | None = None) -> list[str]:
         if run_name:
             rts = f',run_name="{run_name}"'
         else:
@@ -181,8 +185,9 @@ class PlateSetup:
             **kwargs,
         )
 
-    def update_xml(self, root: ET.Element):
+    def update_xml(self, root: ET.Element) -> None:
         samplemap = root.find("FeatureMap/Feature/Id[.='sample']/../..")
+        e: Optional[ET.Element]
         if not samplemap:
             e = ET.SubElement(root, "FeatureMap")
             v = ET.SubElement(e, "Feature")
