@@ -1,13 +1,15 @@
 from __future__ import annotations
+
 import asyncio
-from asyncio.futures import Future
+import io
 import logging
 import re
-import io
-from dataclasses import dataclass
-from typing import Any, Coroutine, Literal, Optional, Protocol
 import time
+from asyncio.futures import Future
+from dataclasses import dataclass
+from typing import Any, Coroutine, Optional, Protocol, Type
 
+from .scpi_commands import AccessLevel, SCPICommand
 
 NL_OR_Q = re.compile(rb"(?:\n|<(/?)([\w.]+)[ *]*>)")
 Q_ONLY = re.compile(rb"<(/?)([\w.]+)[ *]*>")
@@ -16,24 +18,139 @@ TIMESTAMP = re.compile(rb"(\d{8,}\.\d{3})")
 log = logging.getLogger(__name__)
 
 
+def _validate_command_format(commandstring: bytes) -> None:
+    # This is meant to validate that the command will not mess up comms
+    # The command may be completely malformed otherwise
+
+    tagseq: list[tuple[bytes, bytes, bytes]] = re.findall(
+        rb"<(/?)([\w.]+)[ *]*>|(\n)", commandstring.rstrip()
+    )  # tuple of close?,tag,newline?
+
+    tagstack: list[bytes] = []
+    for c, t, n in tagseq:
+        if n and not tagstack:
+            raise ValueError("newline outside of quotation")
+        elif t and not c:
+            tagstack.append(t)
+        elif c:
+            if not tagstack:
+                raise ValueError(f"unbalanced tag <{c.decode()}{t.decode()}>")
+            opentag = tagstack.pop()
+            if opentag != t:
+                raise ValueError(
+                    f"unbalanced tags <{opentag.decode()}> <{c.decode()}{t.decode()}>"
+                )
+        elif n:
+            continue
+        else:
+            raise ValueError("Unknown")
+    if tagstack:
+        raise ValueError("Unclosed tags")
+
+
 class Error(Exception):
     pass
 
 
-@dataclass
 class CommandError(Error):
+    @staticmethod
+    def parse(command: str, ref_index: str, response: str) -> CommandError:
+        m = re.match(r"\[(\w+)\] (.*)", response)
+        if (not m) or (m[1] not in COM_ERRORS):
+            return UnparsedCommandError(command, ref_index, response)
+        try:
+            return COM_ERRORS[m[1]].parse(command, ref_index, m[2])
+        except ValueError:
+            return UnparsedCommandError(command, ref_index, response)
+
+
+@dataclass
+class UnparsedCommandError(CommandError):
     command: Optional[str]
     ref_index: Optional[str]
     response: str
 
 
+@dataclass
+class InsufficientAccess(CommandError):
+    command: str
+    requiredAccess: AccessLevel
+    currentAccess: AccessLevel
+    message: str
+
+    @classmethod
+    def parse(cls, command: str, ref_index: str, message: str) -> InsufficientAccess:
+        m = re.match(
+            r'-requiredAccess="(\w+)" -currentAccess="(\w+)" --> (.*)', message
+        )
+        if not m:
+            raise ValueError
+        return cls(command, AccessLevel(m[1]), AccessLevel(m[2]), m[3])
+
+
+@dataclass
+class AuthError(CommandError):
+    command: str
+    message: str
+
+    @classmethod
+    def parse(cls, command: str, ref_index: str, message: str) -> AuthError:
+        m = re.match(r"--> (.*)", message)
+        if not m:
+            raise ValueError
+        return cls(command, m[1])
+
+
+@dataclass
+class AccessLevelExceeded(CommandError):
+    command: str
+    accessLimit: AccessLevel
+    message: str
+
+    @classmethod
+    def parse(cls, command: str, ref_index: str, message: str) -> AccessLevelExceeded:
+        m = re.match(r'-accessLimit="(\w+)" --> (.*)', message)
+        if not m:
+            raise ValueError
+        return cls(command, AccessLevel(m[1]), m[2])
+
+
+@dataclass
+class InvocationError(CommandError):
+    command: str
+    message: str
+
+    @classmethod
+    def parse(cls, command: str, ref_index: str, message: str) -> InvocationError:
+        return cls(command, message)
+
+
+@dataclass
+class NoMatch(CommandError):
+    command: str
+    message: str
+
+    @classmethod
+    def parse(cls, command: str, ref_index: str, message: str) -> NoMatch:
+        return cls(command, message)
+
+
+COM_ERRORS: dict[str, Type[CommandError]] = {
+    "InsufficientAccess": InsufficientAccess,
+    "AuthError": AuthError,
+    "AccessLevelExceeded": AccessLevelExceeded,
+    "InvocationError": InvocationError,
+    "NoMatch": NoMatch,
+}
+
+
 class ReplyError(IOError):
     pass
 
 
 class SubHandler(Protocol):
     def __call__(
-        self, topic: bytes, message: bytes, timestamp: float | None = None
+        self, topic: bytes, message: bytes, timestamp: float | None
     ) -> Coroutine[None, None, None]:  # pragma: no cover
         ...
 
@@ -41,10 +158,17 @@ class SubHandler(Protocol):
 class QS_IS_Protocol(asyncio.Protocol):
     lostconnection: Future[Any]
     last_received: float
+    waiting_commands: list[
+        tuple[
+            bytes,
+            None
+            | Future[tuple[bytes, bytes, None | Future[tuple[bytes, bytes, None]]]],
+        ]
+    ]
 
     def __init__(self) -> None:
         self.default_topic_handler = self._default_topic_handler
-        self.readymsg = asyncio.get_running_loop().create_future()
+        self.readymsg: Future[str] = asyncio.get_running_loop().create_future()
         self.lostconnection = asyncio.get_running_loop().create_future()
         self.should_be_connected = False
         self.last_received = time.time()
@@ -73,17 +197,7 @@ class QS_IS_Protocol(asyncio.Protocol):
         self.should_be_connected = True
         # setup connection.
         self.transport = transport
-        self.waiting_commands: list[
-            tuple[
-                bytes,
-                None
-                | Future[
-                    tuple[
-                        bytes, None | bytes, None | asyncio.Future[tuple[bytes, bytes]]
-                    ]
-                ],
-            ]
-        ] = []
+        self.waiting_commands = []
         self.buffer = io.BytesIO()
         self.quote_stack: list[bytes] = []
         self.topic_handlers: dict[bytes, SubHandler] = {}
@@ -180,25 +294,33 @@ class QS_IS_Protocol(asyncio.Protocol):
 
     async def run_command(
         self,
-        comm: str | bytes,
+        comm: str | bytes | SCPICommand,
         ack_timeout: int = 300,
         just_ack: bool = True,
         uid: bool = True,
     ) -> bytes:
         if isinstance(comm, str):
             comm = comm.encode()
+        elif isinstance(comm, SCPICommand):
+            comm = comm.to_string().encode()
+        comm = comm.rstrip()
+        _validate_command_format(comm)
         log.debug(f"Running command {comm.decode()}")
         loop = asyncio.get_running_loop()
 
-        comfut = loop.create_future()
+        comfut: Future[
+            tuple[bytes, bytes, None | Future[tuple[bytes, bytes, None]]]
+        ] = loop.create_future()
         if uid:
             import random
 
             commref = str(random.randint(1, 2 ** 30)).encode()
-            comm = commref + b" " + comm
-        self.transport.write((comm + b"\n"))
-        log.debug(f"Sent command {comm!r}")
-        if m := re.match(rb"^(\d+) ", comm):
+            comm_with_ref = commref + b" " + comm
+        else:
+            comm_with_ref = comm
+        self.transport.write((comm_with_ref + b"\n"))
+        log.debug(f"Sent command {comm_with_ref!r}")
+        if m := re.match(rb"^(\d+) ", comm_with_ref):
             commref = m[1]
         else:
             commref = comm
@@ -217,6 +339,7 @@ class QS_IS_Protocol(asyncio.Protocol):
                 # self.waiting_commands.append((commref, None))
                 return b""
             else:
+                assert comnext is not None
                 await comnext
                 state, msg, comnext2 = comnext.result()
                 assert comnext2 is None
@@ -225,10 +348,10 @@ class QS_IS_Protocol(asyncio.Protocol):
         if state == b"OK":
             return msg
         elif state == b"ERRor":
-            raise CommandError(
+            raise CommandError.parse(
                 comm.decode(), commref.decode(), msg.decode().rstrip()
             ) from None
         else:  # pragma: no cover
-            raise CommandError(
+            raise CommandError.parse(
                 comm.decode(), commref.decode(), (state + b" " + msg).decode()
             )
