@@ -1,6 +1,7 @@
-from qslib.base import AccessLevel
 import pytest
 
+from qslib.scpi_commands import AccessLevel
+
 levels = ["Guest", "Observer", "Controller", "Administrator", "Full"]
 
 invalid = 5
